@@ -1,10 +1,6 @@
-import hashlib
 import logging
 import json
 from os import path, makedirs
-import os
-import shutil
-import tempfile
 from urllib.parse import urljoin, urlparse
 import importlib.util
 
@@ -15,15 +11,17 @@ from n2y.errors import (
     UseNextClass, is_api_error_code, APIErrorCode
 )
 from n2y.file import File
+from n2y.emoji import Emoji
 from n2y.page import Page
 from n2y.database import Database
+from n2y.comment import Comment
 from n2y.blocks import DEFAULT_BLOCKS
 from n2y.properties import DEFAULT_PROPERTIES
 from n2y.property_values import DEFAULT_PROPERTY_VALUES
 from n2y.user import User
 from n2y.rich_text import DEFAULT_RICH_TEXTS, RichTextArray
 from n2y.mentions import DEFAULT_MENTIONS
-from n2y.utils import strip_hyphens
+from n2y.utils import sanitize_filename, strip_hyphens
 
 
 DEFAULT_NOTION_CLASSES = {
@@ -34,9 +32,11 @@ DEFAULT_NOTION_CLASSES = {
     "property_values": DEFAULT_PROPERTY_VALUES,
     "user": User,
     "file": File,
+    "emoji": Emoji,
     "rich_text_array": RichTextArray,
     "rich_texts": DEFAULT_RICH_TEXTS,
     "mentions": DEFAULT_MENTIONS,
+    "comment": Comment,
 }
 
 
@@ -49,10 +49,9 @@ class Client:
     """
     An instance of the client class has a few purposes:
 
-    1. To store configuration
-    2. To retrieve data from Notion
-    3. To determine what classes to use to wrap this notion data, based on the configuration
-    4. To act as a shared global store for all of the objects that are pulled
+    1. To retrieve data from Notion
+    2. To determine what classes to use to wrap this notion data
+    3. To act as a shared global store for all of the objects that are pulled
        from Notion.
 
     In particular there is a cache of all pages and databases which ensure that
@@ -66,32 +65,21 @@ class Client:
         media_root='.',
         media_url='',
         plugins=None,
-        content_property=None,
-        id_property=None,
-        url_property=None,
-        filename_property=None,
-        database_config=None,
     ):
         self.access_token = access_token
         self.media_root = media_root
         self.media_url = media_url
-        self.content_property = content_property
-        self.id_property = id_property
-        self.url_property = url_property
-        self.filename_property = filename_property
-        self.database_config = database_config if database_config is not None else {}
 
         self.base_url = "https://api.notion.com/v1/"
         self.headers = {
             "Authorization": f"Bearer {self.access_token}",
             "Content-Type": "application/json",
-            "Notion-Version": "2022-02-22",
+            "Notion-Version": "2022-06-28",
         }
 
         self.databases_cache = {}
         self.pages_cache = {}
 
-        self.notion_classes = self.get_default_classes()
         self.load_plugins(plugins)
         self.plugin_data = {}
 
@@ -105,6 +93,7 @@ class Client:
         return notion_classes
 
     def load_plugins(self, plugins):
+        self.notion_classes = self.get_default_classes()
         if plugins is not None:
             for plugin in plugins:
                 plugin_module = importlib.import_module(plugin)
@@ -208,6 +197,9 @@ class Client:
     def wrap_notion_file(self, notion_data):
         return self.instantiate_class("file", None, self, notion_data)
 
+    def wrap_notion_emoji(self, notion_data):
+        return self.instantiate_class("emoji", None, self, notion_data)
+
     def wrap_notion_rich_text_array(self, notion_data, block=None):
         return self.instantiate_class("rich_text_array", None, self, notion_data, block)
 
@@ -258,25 +250,18 @@ class Client:
             self.databases_cache[database_id] = database
         return database
 
-    def get_database_pages(self, database_id):
-        notion_pages = self.get_database_notion_pages(database_id)
+    def get_database_pages(self, database_id, filter=None, sorts=None):
+        notion_pages = self.get_database_notion_pages(database_id, filter, sorts)
         return [self._wrap_notion_page(np) for np in notion_pages]
 
-    def get_database_notion_pages(self, database_id):
-        results = []
+    def get_database_notion_pages(self, database_id, filter, sorts):
         url = f"{self.base_url}databases/{database_id}/query"
-        request_data = self._create_database_request_data(database_id)
-        while True:
-            data = self._post_url(url, request_data)
-            results.extend(data["results"])
-            if not data["has_more"]:
-                return results
-            else:
-                request_data["start_cursor"] = data["next_cursor"]
-
-    def _create_database_request_data(self, database_id):
-        stripped_database_id = strip_hyphens(database_id)
-        return self.database_config.get(stripped_database_id, {})
+        request_data = {}
+        if filter:
+            request_data["filter"] = filter
+        if sorts:
+            request_data["sorts"] = sorts
+        return self._paginated_request(self._post_url, url, request_data)
 
     def get_page(self, page_id):
         """
@@ -309,15 +294,15 @@ class Client:
 
     def get_child_notion_blocks(self, block_id):
         url = f"{self.base_url}blocks/{block_id}/children"
-        params = {}
-        results = []
-        while True:
-            data = self._get_url(url, params)
-            results.extend(data["results"])
-            if not data["has_more"]:
-                return results
-            else:
-                params["start_cursor"] = data["next_cursor"]
+        return self._paginated_request(self._get_url, url, {})
+
+    def get_comments(self, block_id):
+        url = f"{self.base_url}comments"
+        comments = self._paginated_request(self._get_url, url, {"block_id": block_id})
+        return [self.wrap_notion_comment(nd) for nd in comments]
+
+    def wrap_notion_comment(self, notion_data):
+        return self.instantiate_class("comment", None, self, notion_data)
 
     def get_page_property(self, page_id, property_id):
         notion_property = self.get_notion_page_property(page_id, property_id)
@@ -340,6 +325,17 @@ class Client:
         response = requests.post(url, headers=self.headers, json=data)
         return self._parse_response(response)
 
+    def _paginated_request(self, request_method, url, initial_params):
+        params = initial_params
+        results = []
+        while True:
+            data = request_method(url, params)
+            results.extend(data["results"])
+            if not data["has_more"]:
+                return results
+            else:
+                params["start_cursor"] = data["next_cursor"]
+
     def _parse_response(self, response):
         """Taken from https://github.com/ramnes/notion-sdk-py"""
         try:
@@ -366,30 +362,30 @@ class Client:
         """
         url_path = path.basename(urlparse(url).path)
         _, extension = path.splitext(url_path)
-        with requests.get(url, stream=True) as request_stream:
-            content_iterator = request_stream.iter_content(4096)
-            return self.save_file(content_iterator, page, extension)
+        request_stream = requests.get(url, stream=True)
+        return self.save_file(request_stream.content, page, extension)
 
-    def save_file(self, content_iterator, page, extension):
-        """
-        Save the content in the provided iterator into a file in MEDIA_ROOT. The
-        file name is determined from the page name, file extension, and an md5
-        hash of the content. The md5 hash is calculated as the data is streamed
-        to a temporary file, which is then moved to the final location once the
-        md5 hash can be calculated.
-        """
-        temp_fd, temp_filepath = tempfile.mkstemp()
-        hash_md5 = hashlib.md5()
-        with os.fdopen(temp_fd, 'wb') as temp_file:
-            for chunk in content_iterator:
-                hash_md5.update(chunk)
-                temp_file.write(chunk)
-
-        num_hash_characters = 8  # just long enough to avoid collisions
-        hash = hash_md5.hexdigest()[:num_hash_characters]
-        relative_filepath = "".join([page.filename, "-", hash, extension])
+    def save_file(self, content, page, extension):
+        page_id_chars = strip_hyphens(page.notion_id)
+        page_title = sanitize_filename(page.title.to_plain_text())
+        relative_filepath = f"{page_title}-{page_id_chars[:11]}{extension}"
         full_filepath = path.join(self.media_root, relative_filepath)
-
-        makedirs(path.dirname(full_filepath), exist_ok=True)
-        shutil.move(temp_filepath, full_filepath)
+        makedirs(self.media_root, exist_ok=True)
+        with open(full_filepath, 'wb') as temp_file:
+            temp_file.write(content)
         return urljoin(self.media_url, relative_filepath)
+
+    def append_block_children(self, block_id, children):
+        response = requests.patch(
+            f"{self.base_url}blocks/{block_id}/children",
+            json={"children": children}, headers=self.headers
+        )
+        return self._parse_response(response)
+
+    def delete_block(self, block_id):
+        headers = {**self.headers}
+        del headers['Content-Type']
+        response = requests.delete(
+            f"{self.base_url}blocks/{block_id}", headers=headers
+        )
+        return self._parse_response(response)
